from .client import FuelCheckClient
from .dto import (
    AveragePrice, Variance, Station, Period, Price, FuelCheckError,
    GetFuelPricesResponse, FuelType, GetReferenceDataResponse,
    SortField, TrendPeriod
)

__all__ = ["FuelCheckClient", "AveragePrice", "Variance", "Station", "Period",
           "Price", "FuelCheckError", "GetFuelPricesResponse", "FuelType",
           "GetReferenceDataResponse", "SortField", "TrendPeriod"]
__version__ = "1.1.2"
